"""Constants for the Gree Climate integration."""

COORDINATORS = "coordinators"

DATA_DISCOVERY_SERVICE = "gree_discovery"
DATA_DISCOVERY_INTERVAL = "gree_discovery_interval"

DISCOVERY_SCAN_INTERVAL = 300
DISCOVERY_TIMEOUT = 8
DISPATCH_DEVICE_DISCOVERED = "gree_device_discovered"
DISPATCHERS = "dispatchers"

DOMAIN = "gree"
COORDINATOR = "coordinator"

FAN_MEDIUM_LOW = "medium low"
FAN_MEDIUM_HIGH = "medium high"

MAX_ERRORS = 2

TARGET_TEMPERATURE_STEP = 1
