"""Constants for the xbox integration."""

DOMAIN = "xbox"

OAUTH2_AUTHORIZE = "https://login.live.com/oauth20_authorize.srf"
OAUTH2_TOKEN = "https://login.live.com/oauth20_token.srf"

EVENT_NEW_FAVORITE = "xbox/new_favorite"
