"""Constants for the Goal Zero Yeti integration."""
from datetime import timedelta

ATTRIBUTION = "Data provided by Goal Zero"
ATTR_DEFAULT_ENABLED = "default_enabled"

DATA_KEY_COORDINATOR = "coordinator"
DOMAIN = "goalzero"
DEFAULT_NAME = "Yeti"
DATA_KEY_API = "api"
MANUFACTURER = "Goal Zero"
MIN_TIME_BETWEEN_UPDATES = timedelta(seconds=30)
