"""Constants for the FiveM integration."""

ATTR_PLAYERS_LIST = "players_list"
ATTR_RESOURCES_LIST = "resources_list"

DOMAIN = "fivem"

ICON_PLAYERS_MAX = "mdi:account-multiple"
ICON_PLAYERS_ONLINE = "mdi:account-multiple"
ICON_RESOURCES = "mdi:playlist-check"

MANUFACTURER = "Cfx.re"

NAME_PLAYERS_MAX = "Players Max"
NAME_PLAYERS_ONLINE = "Players Online"
NAME_RESOURCES = "Resources"
NAME_STATUS = "Status"

SCAN_INTERVAL = 60

UNIT_PLAYERS_MAX = "players"
UNIT_PLAYERS_ONLINE = "players"
UNIT_RESOURCES = "resources"
